function [varargout] = vsFindDelayRelation(data, lag, method, ...
    binVec,verbose)
% vsFindDelayRelation - function to find relation matrix and delay matrix
%
% Syntax:  [relMat delayMat miAll] = ...
%             vsFindDelayRelation(data, lag, method)
%
% Inputs:
%    data   - data for learning the relationships and time delay, in the 
%    form of data(i,m) where i is the node index and m is case index
%    lag    - maximum lag between data allowed
%    method - 'tdmi'
%    binVec - vector that stores the bin numbers
%
%
% Outputs:
%    relMat   - relation matrix
%    delayMat - time delay matrix
%    miAll    - mi function of every node-pair
%
%
% Author: Chen Change Loy
%
%---------------------------- BEGIN CODE ----------------------------------

if nargin < 5
    verbose = 0;
end

% Compute the total number of nodes
totalNode = size(data,1);

% Initialisation
relMat = zeros(totalNode);
delayMat = zeros(totalNode);
if nargout == 3
    miAll = cell(totalNode);
end

for nNodei = 1:(totalNode-1)
    for nNodej = (nNodei+1):totalNode
        if verbose
            fprintf('i = %d, j = %d\n',nNodei,nNodej);
        end

        seqi = data(nNodei,:);
        seqj = data(nNodej,:);

        switch method
            case 'tdmi'
                % Compute the TDMI
                if nargin > 3
                    noBinX = binVec(nNodei);
                    noBinY = binVec(nNodej);
                    
                    [mi,lag] = vsTDMI(seqi, seqj, lag,noBinX,noBinY);
                    
                else
                    [mi,lag] = vsTDMI(seqi, seqj, lag);
                end

                [miMaxValue miMaxIndex] = max(mi);

                if nargout == 3
                    miAll{nNodei,nNodej} = mi;
                end

                relMat(nNodei,nNodej) = miMaxValue;
                relMat(nNodej,nNodei) = relMat(nNodei,nNodej);

                % Estimate the delay
                delayMat(nNodei,nNodej) = ceil(length(lag)/2) - miMaxIndex;
                delayMat(nNodej,nNodei) = -delayMat(nNodei,nNodej);
            otherwise
                error('Invalid scoring function.');
        end
    end
end

switch nargout
    case 2
        varargout(1) = {relMat};
        varargout(2) = {delayMat};
    case 3
        varargout(1) = {relMat};
        varargout(2) = {delayMat};
        varargout(3) = {miAll};
    otherwise
        error('Invalid number of output argument');
end