function [mi,lag] = vsTDMI(x,y,lag,noBinX,noBinY)
% vsTDMI - compute time delay mutual information between x and y
%
%
%
% Syntax:  [mi,lag] = vsTDMI(x,y,lag,noBinX,noBinY)
%
% Inputs:
%    x, y           - 1-dimensional vector
%    lag            - possible time delay range between x and y
%    noBinX, noBinY - number of bins (or number of possible levels) in x
%    and y
%
% Outputs:
%    mi  - the mutual information function
%    lag - time delay range between x and y
%
% Examples:
% [mi,lag] = vsTDMI(x,y,lag,noBinX,noBinY)
% [miMaxValue miMaxIndex] = max(mi);
% delay = ceil(length(lag)/2) - miMaxIndex
%
% Original: Aslak Grinsted
% Modified: Chen Change Loy
%
%---------------------------- BEGIN CODE ----------------------------------

x = x(:);
y = y(:);
n = length(x);
if n ~= length(y)
    error('Vectors x and y should have same length.');
end

mi = zeros(size(lag));
lastbins = NaN;

% Normalisation
x = x - min(x);
if sum(x) == 0
    return;
end
x = x*(1-eps)/max(x);

y = y - min(y);
if sum(y) == 0
    return;
end
y = y*(1-eps)/max(y);

for ii = 1:length(lag)
    
    abslag = abs(lag(ii));
    
    if nargin < 4 % Determine the number of bins automatically
        bins = floor( 1 + log2(n-abslag) + 0.5);
        
        if bins ~= lastbins
            % Assign x and y to bins
            binx = floor(x*bins) + 1;
            biny = floor(y*bins) + 1;
        end
        lastbins = bins;
        
        Pxy = zeros(bins);
    else % Define own number of bins
        binx = floor(x*noBinX) + 1;
        biny = floor(y*noBinY) + 1;
        
        Pxy = zeros(noBinX,noBinY);
    end
    
    for jj = 1:(n - abslag)
        kk = jj + abslag;
        if lag(ii) < 0
            % Swapping
            temp = jj;
            jj = kk;
            kk = temp;
        end
        Pxy(binx(kk),biny(jj)) = Pxy(binx(kk),biny(jj)) + 1;
    end
    
    Pxy = Pxy / (n-abslag);
    Pxy = Pxy + eps; % avoid division and log of zero
    Px = sum(Pxy,2);
    Py = sum(Pxy,1);
    
    Hx = -sum(Px.*log2(Px));
    Hy = -sum(Py.*log2(Py));
    Hxy = -sum(Pxy(:).*log2(Pxy(:)));
    mi(ii) = Hx + Hy - Hxy;
end
